<?php

namespace Util;

/**
 * Handles Tokens Validations/Generations
 * 
 * An static class that handles JWT Token Generation and Validation,
 * including static methods to encode and decode the Token.
 * 
 * @author Rômulo Alves Lousada
 */
class JWT 
{
    /**
     * Token Key
     *
     * Random generated string that will be used as secret key to generate
     * the tokens and later validations.
     *
     * @var string
     */
    const SECRET_KEY = "9DZZPVdW2wEeCJOMGutCAmUjZu4Adqx7GpZ9eBkpKGPSJp9HKgOgzBbftkKvlsOm";

    /**
     * Creates the Token
     * 
     * Creates the Token based on the information
     * sent on payload, adding the header that is already
     * formatted using the encryption type.
     *
     * @param array $payload - Array with data that will be used to generated the token
     * 
     * @return string
     * 
     * @access public
     * @static
     * @author Rômulo Alves Lousada
     */
    public static function generateToken($payload) 
    {
        $header= array(
            "typ" => "JWT",
            "alg" => "HS256"
        );

        $objectHeaderJSON = json_encode((object) $header);
        $objectPayloadJSON = json_encode((object) $payload);
        
        $unsignedToken = self::base64_url_encode($objectHeaderJSON) . "." . self::base64_url_encode($objectPayloadJSON);
        
        return $unsignedToken . "." . self::sign($unsignedToken);
    }

    /**
     * Validates token expiration time
     * 
     * Validate the expiration time of the Token
     * if the "exp" attribute is set.
     *
     * @param string $token Base64 URL Encoded Token
     * 
     * @return bool
     * @throws Exception [if the expiration time is lower than the current unix time]
     * 
     * @access public
     * @static
     * @author Rômulo Alves Lousada
     */
    public static function validateExpiration($token) 
    {
        try {
            $payload = self::getTokenData($token);
            $expiration = $payload["exp"] ? $payload["exp"] : 0 ;

            if ($expiration != 0 && $expiration < time()) {
                throw new \Exception();
            }

            return true;
        }
        catch (\Exception $e) {
            return false;
        }
    }

    /**
     * Validates token integrity
     * 
     * Validate the contents of the Token to check
     * if it's a valid JWT Token generated on Login.
     *
     * @param string $token - Base64 URL Encoded Token
     * 
     * @return bool
     * @throws Exception if the sign of the content isn't equal to the signature
     * 
     * @access public
     * @static
     * @author Rômulo Alves Lousada
     */
    public static function validateToken($token) 
    {
        try {
            $sections  = explode(".", $token);
            $content   = $sections[0] . "." . $sections[1];
            $signature = $sections[2];

            if (self::sign($content) != $signature) {
                throw new \Exception();
            }

            return true;
        }
        catch (\Exception $e) {
            return false;
        }
    }

    /**
     * Returns token payload
     * 
     * Received the Base64 URL Encoded Token and transforms
     * it into an array with the payload data used to generate it.
     *
     * @param string $token - Base64 URL Encoded Token
     * 
     * @return array
     * 
     * @access private
     * @static
     * @author Rômulo Alves Lousada
     */
    private static function getTokenData($token) 
    {
        $segments = explode('.', $token);
        list($headb64, $bodyb64, $cryptob64) = $segments;
        return json_decode(self::base64_url_decode($bodyb64), true);
    }

    /**
     * Generates Token Signature
     * 
     * Hash the Header + the Payload data sent using the secret key
     * to generate the Signature.
     *
     * @param string $data - Contains the base64 URL Encoded string 
     *                       of the value from the header + the payload.
     * 
     * @return string
     * 
     * @access private
     * @static
     * @author Rômulo Alves Lousada
     */
    private static function sign($data) 
    {
        return self::base64_url_encode(hash_hmac("sha256", $data, self::SECRET_KEY, true));
    }

    /**
     * Encodes string using base64
     * 
     * Remove "=" signals and encode the string passed
     * using base64_encode PHP Function.
     *
     * @param string $value - Value to be encoded. 
     * 
     * @return string
     * 
     * @access private
     * @static
     * @author Rômulo Alves Lousada
     */
    private static function base64_url_encode($value) 
    {
        return str_replace('=', '', strtr(base64_encode($value), '+/', '-_'));
    }

    /**
     * Decodes base64 string
     * 
     * Decodes an encoded base64 string and returns it's value
     *
     * @param string $value - Value to be decoded
     * 
     * @return string
     * 
     * @access private
     * @static
     * @author Rômulo Alves Lousada
     */
    private static function base64_url_decode($value) 
    {
        return base64_decode(strtr($value, '-_', '+/'));
    }
}